@@ -24,6 +24,7 @@
 #include "NVCtrl.h"
 #include <GL/glx.h> /* GLX #defines */
 #include <EGL/egl.h>
+#include <vulkan/vulkan.h>
 #include <X11/extensions/Xrandr.h> /* Xrandr */
 #include <nvml.h>
 
@@ -164,44 +165,48 @@ struct __NvCtrlNvmlAttributes {
     struct {
         void *handle;
 
-        typeof(nvmlInit)                                (*init);
-        typeof(nvmlShutdown)                            (*shutdown);
-        typeof(nvmlDeviceGetHandleByIndex)              (*deviceGetHandleByIndex);
-        typeof(nvmlDeviceGetUUID)                       (*deviceGetUUID);
-        typeof(nvmlDeviceGetCount)                      (*deviceGetCount);
-        typeof(nvmlDeviceGetTemperature)                (*deviceGetTemperature);
-        typeof(nvmlDeviceGetName)                       (*deviceGetName);
-        typeof(nvmlDeviceGetVbiosVersion)               (*deviceGetVbiosVersion);
-        typeof(nvmlDeviceGetMemoryInfo)                 (*deviceGetMemoryInfo);
-        typeof(nvmlDeviceGetMemoryInfo_v2)              (*deviceGetMemoryInfo_v2);
-        typeof(nvmlDeviceGetPciInfo)                    (*deviceGetPciInfo);
-        typeof(nvmlDeviceGetCurrPcieLinkWidth)          (*deviceGetCurrPcieLinkWidth);
-        typeof(nvmlDeviceGetMaxPcieLinkGeneration)      (*deviceGetMaxPcieLinkGeneration);
-        typeof(nvmlDeviceGetMaxPcieLinkWidth)           (*deviceGetMaxPcieLinkWidth);
-        typeof(nvmlDeviceGetVirtualizationMode)         (*deviceGetVirtualizationMode);
-        typeof(nvmlDeviceGetGridLicensableFeatures_v4)  (*deviceGetGridLicensableFeatures);
-        typeof(nvmlDeviceGetGspFirmwareMode)            (*deviceGetGspFirmwareMode);
-        typeof(nvmlDeviceGetUtilizationRates)           (*deviceGetUtilizationRates);
-        typeof(nvmlDeviceGetTemperatureThreshold)       (*deviceGetTemperatureThreshold);
-        typeof(nvmlDeviceGetFanSpeed_v2)                (*deviceGetFanSpeed_v2);
-        typeof(nvmlSystemGetDriverVersion)              (*systemGetDriverVersion);
-        typeof(nvmlDeviceGetEccMode)                    (*deviceGetEccMode);
-        typeof(nvmlDeviceGetDefaultEccMode)             (*deviceGetDefaultEccMode);
-        typeof(nvmlDeviceSetEccMode)                    (*deviceSetEccMode);
-        typeof(nvmlDeviceGetTotalEccErrors)             (*deviceGetTotalEccErrors);
-        typeof(nvmlDeviceClearEccErrorCounts)           (*deviceClearEccErrorCounts);
-        typeof(nvmlDeviceGetMemoryErrorCounter)         (*deviceGetMemoryErrorCounter);
-        typeof(nvmlSystemGetNVMLVersion)                (*systemGetNVMLVersion);
-        typeof(nvmlDeviceGetNumGpuCores)                (*deviceGetNumGpuCores);
-        typeof(nvmlDeviceGetMemoryBusWidth)             (*deviceGetMemoryBusWidth);
-        typeof(nvmlDeviceGetIrqNum)                     (*deviceGetIrqNum);
-        typeof(nvmlDeviceGetPowerSource)                (*deviceGetPowerSource);
-        typeof(nvmlDeviceGetNumFans)                    (*deviceGetNumFans);
-        typeof(nvmlDeviceSetFanSpeed_v2)                (*deviceSetFanSpeed_v2);
-        typeof(nvmlDeviceGetTargetFanSpeed)             (*deviceGetTargetFanSpeed);
-        typeof(nvmlDeviceGetMinMaxFanSpeed)             (*deviceGetMinMaxFanSpeed);
-        typeof(nvmlDeviceSetFanControlPolicy)           (*deviceSetFanControlPolicy);
-        typeof(nvmlDeviceGetFanControlPolicy_v2)        (*deviceGetFanControlPolicy_v2);
+        typeof(nvmlInit)                                (*Init);
+        typeof(nvmlShutdown)                            (*Shutdown);
+        typeof(nvmlDeviceGetHandleByIndex)              (*DeviceGetHandleByIndex);
+        typeof(nvmlDeviceGetUUID)                       (*DeviceGetUUID);
+        typeof(nvmlDeviceGetCount)                      (*DeviceGetCount);
+        typeof(nvmlDeviceGetTemperature)                (*DeviceGetTemperature);
+        typeof(nvmlDeviceGetName)                       (*DeviceGetName);
+        typeof(nvmlDeviceGetVbiosVersion)               (*DeviceGetVbiosVersion);
+        typeof(nvmlDeviceGetMemoryInfo)                 (*DeviceGetMemoryInfo);
+        typeof(nvmlDeviceGetMemoryInfo_v2)              (*DeviceGetMemoryInfo_v2);
+        typeof(nvmlDeviceGetPciInfo)                    (*DeviceGetPciInfo);
+        typeof(nvmlDeviceGetCurrPcieLinkWidth)          (*DeviceGetCurrPcieLinkWidth);
+        typeof(nvmlDeviceGetMaxPcieLinkGeneration)      (*DeviceGetMaxPcieLinkGeneration);
+        typeof(nvmlDeviceGetMaxPcieLinkWidth)           (*DeviceGetMaxPcieLinkWidth);
+        typeof(nvmlDeviceGetVirtualizationMode)         (*DeviceGetVirtualizationMode);
+        typeof(nvmlDeviceGetGridLicensableFeatures)     (*DeviceGetGridLicensableFeatures);
+        typeof(nvmlDeviceGetGspFirmwareMode)            (*DeviceGetGspFirmwareMode);
+        typeof(nvmlDeviceGetUtilizationRates)           (*DeviceGetUtilizationRates);
+        typeof(nvmlDeviceGetTemperatureThreshold)       (*DeviceGetTemperatureThreshold);
+        typeof(nvmlDeviceGetFanSpeed_v2)                (*DeviceGetFanSpeed_v2);
+        typeof(nvmlSystemGetDriverVersion)              (*SystemGetDriverVersion);
+        typeof(nvmlDeviceGetEccMode)                    (*DeviceGetEccMode);
+        typeof(nvmlDeviceGetDefaultEccMode)             (*DeviceGetDefaultEccMode);
+        typeof(nvmlDeviceSetEccMode)                    (*DeviceSetEccMode);
+        typeof(nvmlDeviceGetTotalEccErrors)             (*DeviceGetTotalEccErrors);
+        typeof(nvmlDeviceClearEccErrorCounts)           (*DeviceClearEccErrorCounts);
+        typeof(nvmlDeviceGetMemoryErrorCounter)         (*DeviceGetMemoryErrorCounter);
+        typeof(nvmlSystemGetNVMLVersion)                (*SystemGetNVMLVersion);
+        typeof(nvmlDeviceGetNumGpuCores)                (*DeviceGetNumGpuCores);
+        typeof(nvmlDeviceGetMemoryBusWidth)             (*DeviceGetMemoryBusWidth);
+        typeof(nvmlDeviceGetIrqNum)                     (*DeviceGetIrqNum);
+        typeof(nvmlDeviceGetPowerSource)                (*DeviceGetPowerSource);
+        typeof(nvmlDeviceGetNumFans)                    (*DeviceGetNumFans);
+        typeof(nvmlDeviceSetFanSpeed_v2)                (*DeviceSetFanSpeed_v2);
+        typeof(nvmlDeviceGetTargetFanSpeed)             (*DeviceGetTargetFanSpeed);
+        typeof(nvmlDeviceGetMinMaxFanSpeed)             (*DeviceGetMinMaxFanSpeed);
+        typeof(nvmlDeviceSetFanControlPolicy)           (*DeviceSetFanControlPolicy);
+        typeof(nvmlDeviceGetFanControlPolicy_v2)        (*DeviceGetFanControlPolicy_v2);
+        typeof(nvmlDeviceSetDefaultFanSpeed_v2)         (*DeviceSetDefaultFanSpeed_v2);
+        typeof(nvmlDeviceGetPowerUsage)                 (*DeviceGetPowerUsage);
+        typeof(nvmlDeviceGetPowerManagementDefaultLimit)     (*DeviceGetPowerManagementDefaultLimit);
+        typeof(nvmlDeviceGetPowerManagementLimitConstraints) (*DeviceGetPowerManagementLimitConstraints);
 
     } lib;
 
@@ -227,10 +232,15 @@ struct __NvCtrlAttributePrivateHandle {
     Bool glx;                       /* GLX extension available */
     Bool egl;                       /* EGL extension available */
     EGLDisplay egl_dpy;
+    Bool vulkan;                    /* Vulkan extension available */
+    VkInstance *vk_instance;
     NvCtrlXrandrAttributes *xrandr; /* XRandR extension info */
 
     /* NVML-specific attributes */
     NvCtrlNvmlAttributes *nvml;
+
+    /* Wayland display ptr */
+    void *wayland_dpy;
 };
 
 struct __NvCtrlEventPrivateHandle {
@@ -320,6 +330,19 @@ ReturnStatus NvCtrlEglGetVoidAttribute(const NvCtrlAttributePrivateHandle *,
 
 ReturnStatus NvCtrlEglGetStringAttribute(const NvCtrlAttributePrivateHandle *,
                                          unsigned int, int, char **);
+Bool NvCtrlEglDelayedInit(NvCtrlAttributePrivateHandle *);
+
+/* Vulkan extension attribute functions */
+
+Bool NvCtrlInitVkAttributes(NvCtrlAttributePrivateHandle *);
+
+void NvCtrlVkAttributesClose(NvCtrlAttributePrivateHandle *);
+
+ReturnStatus NvCtrlVkGetVoidAttribute(const NvCtrlAttributePrivateHandle *,
+                                      unsigned int, int, void **);
+
+ReturnStatus NvCtrlVkGetStringAttribute(const NvCtrlAttributePrivateHandle *,
+                                        unsigned int, int, char **);
 
 /* XRandR extension attribute functions */
 
